@@ -1,7 +1,5 @@
 # coding: utf-8
 
-from __future__ import absolute_import, print_function
-
 """
 stuff to deal with comments and formatting on dict/list/ordereddict/set
 these are not really related, formatting could be factored out as
@@ -12,55 +10,139 @@ import sys
 import copy
 
 
-from ruamel.yaml.compat import ordereddict, PY2, string_types
+from ruamel.yaml.compat import ordereddict  # type: ignore
+from ruamel.yaml.compat import MutableSliceableSequence, _F, nprintf  # NOQA
 from ruamel.yaml.scalarstring import ScalarString
+from ruamel.yaml.anchor import Anchor
 
-if PY2:
-    from collections import MutableSet, Sized, Set
-else:
-    from collections.abc import MutableSet, Sized, Set
+from collections.abc import MutableSet, Sized, Set, Mapping
 
 if False:  # MYPY
-    from typing import Any, Dict, Optional, List, Union, Optional  # NOQA
+    from typing import Any, Dict, Optional, List, Union, Optional, Iterator  # NOQA
 
 # fmt: off
-__all__ = ["CommentedSeq", "CommentedKeySeq",
-           "CommentedMap", "CommentedOrderedMap",
-           "CommentedSet", 'comment_attrib', 'merge_attrib']
+__all__ = ['CommentedSeq', 'CommentedKeySeq',
+           'CommentedMap', 'CommentedOrderedMap',
+           'CommentedSet', 'comment_attrib', 'merge_attrib',
+           'C_POST', 'C_PRE', 'C_SPLIT_ON_FIRST_BLANK', 'C_BLANK_LINE_PRESERVE_SPACE',
+           ]
 # fmt: on
 
+# splitting of comments by the scanner
+# an EOLC (End-Of-Line Comment) is preceded by some token
+# an FLC (Full Line Comment) is a comment not preceded by a token, i.e. # is
+#   the first non-blank on line
+# a BL is a blank line i.e. empty or spaces/tabs only
+# bits 0 and 1 are combined, you can choose only one
+C_POST = 0b00
+C_PRE = 0b01
+C_SPLIT_ON_FIRST_BLANK = 0b10  # as C_POST, but if blank line then C_PRE all lines before
+# first blank goes to POST even if no following real FLC
+# (first blank -> first of post)
+# 0b11 -> reserved for future use
+C_BLANK_LINE_PRESERVE_SPACE = 0b100
+# C_EOL_PRESERVE_SPACE2 = 0b1000
+
+
+class IDX:
+    # temporary auto increment, so rearranging is easier
+    def __init__(self):
+        self._idx = 0
+
+    def __call__(self):
+        x = self._idx
+        self._idx += 1
+        return x
+
+    def __str__(self):
+        return str(self._idx)
+
+
+cidx = IDX()
+
+# more or less in order of subjective expected likelyhood
+# the _POST and _PRE ones are lists themselves
+C_VALUE_EOL = C_ELEM_EOL = cidx()
+C_KEY_EOL = cidx()
+C_KEY_PRE = C_ELEM_PRE = cidx()  # not this is not value
+C_VALUE_POST = C_ELEM_POST = cidx()  # not this is not value
+C_VALUE_PRE = cidx()
+C_KEY_POST = cidx()
+C_TAG_EOL = cidx()
+C_TAG_POST = cidx()
+C_TAG_PRE = cidx()
+C_ANCHOR_EOL = cidx()
+C_ANCHOR_POST = cidx()
+C_ANCHOR_PRE = cidx()
+
+
 comment_attrib = '_yaml_comment'
 format_attrib = '_yaml_format'
 line_col_attrib = '_yaml_line_col'
-anchor_attrib = '_yaml_anchor'
 merge_attrib = '_yaml_merge'
 tag_attrib = '_yaml_tag'
 
 
-class Comment(object):
-    # sys.getsize tested the Comment objects, __slots__ makes them bigger
+class Comment:
+    # using sys.getsize tested the Comment objects, __slots__ makes them bigger
     # and adding self.end did not matter
-    __slots__ = 'comment', '_items', '_end', '_start'
+    __slots__ = 'comment', '_items', '_post', '_pre'
     attrib = comment_attrib
 
-    def __init__(self):
+    def __init__(self, old=True):
         # type: () -> None
+        self._pre = None if old else []
         self.comment = None  # [post, [pre]]
         # map key (mapping/omap/dict) or index (sequence/list) to a  list of
         # dict: post_key, pre_key, post_value, pre_value
         # list: pre item, post item
         self._items = {}  # type: Dict[Any, Any]
         # self._start = [] # should not put these on first item
-        self._end = []  # type: List[Any] # end of document comments
+        self._post = []  # type: List[Any] # end of document comments
 
     def __str__(self):
         # type: () -> str
-        if bool(self._end):
+        if bool(self._post):
             end = ',\n  end=' + str(self._end)
         else:
             end = ""
         return 'Comment(comment={0},\n  items={1}{2})'.format(self.comment, self._items, end)
 
+    def _old__repr__(self):
+        # type: () -> str
+        if bool(self._post):
+            end = ',\n  end=' + str(self._post)
+        else:
+            end = ""
+        try:
+            ln = max([len(str(k)) for k in self._items]) + 1
+        except ValueError:
+            ln = ''
+        it = '    '.join(
+            ['{:{}} {}\n'.format(str(k) + ':', ln, v) for k, v in self._items.items()]
+        )
+        if it:
+            it = '\n    ' + it + '  '
+        return 'Comment(\n  start={},\n  items={{{}}}{})'.format(self.comment, it, end)
+
+    def __repr__(self):
+        if self._pre is None:
+            return self._old__repr__()
+        if bool(self._post):
+            end = ',\n  end=' + repr(self._post)
+        else:
+            end = ""
+        try:
+            ln = max([len(str(k)) for k in self._items]) + 1
+        except ValueError:
+            ln = ''
+        it = '    '.join(
+            ['{:{}} {}\n'.format(str(k) + ':', ln, v) for k, v in self._items.items()]
+        )
+        if it:
+            it = '\n    ' + it + '  '
+        return 'Comment(\n  pre={},\n  items={{{}}}{})'.format(self.pre, it, end)
+
     @property
     def items(self):
         # type: () -> Any
@@ -69,22 +151,59 @@ class Comment(object):
     @property
     def end(self):
         # type: () -> Any
-        return self._end
+        return self._post
 
     @end.setter
     def end(self, value):
         # type: (Any) -> None
-        self._end = value
+        self._post = value
 
     @property
-    def start(self):
+    def pre(self):
         # type: () -> Any
-        return self._start
+        return self._pre
 
-    @start.setter
-    def start(self, value):
+    @pre.setter
+    def pre(self, value):
         # type: (Any) -> None
-        self._start = value
+        self._pre = value
+
+    def get(self, item, pos):
+        x = self._items.get(item)
+        if x is None or len(x) < pos:
+            return None
+        return x[pos]  # can be None
+
+    def set(self, item, pos, value):
+        x = self._items.get(item)
+        if x is None:
+            self._items[item] = x = [None] * (pos + 1)
+        else:
+            while len(x) <= pos:
+                x.append(None)
+        assert x[pos] is None
+        x[pos] = value
+
+    def __contains__(self, x):
+        # test if a substring is in any of the attached comments
+        if self.comment:
+            if self.comment[0] and x in self.comment[0].value:
+                return True
+            if self.comment[1]:
+                for c in self.comment[1]:
+                    if x in c.value:
+                        return True
+        for value in self.items.values():
+            if not value:
+                continue
+            for c in value:
+                if c and x in c.value:
+                    return True
+        if self.end:
+            for c in self.end:
+                if x in c.value:
+                    return True
+        return False
 
 
 # to distinguish key from None
@@ -93,7 +212,7 @@ def NoComment():
     pass
 
 
-class Format(object):
+class Format:
     __slots__ = ('_flow_style',)
     attrib = format_attrib
 
@@ -120,7 +239,11 @@ class Format(object):
         return self._flow_style
 
 
-class LineCol(object):
+class LineCol:
+    """
+    line and column information wrt document, values start at zero (0)
+    """
+
     attrib = line_col_attrib
 
     def __init__(self):
@@ -159,21 +282,15 @@ class LineCol(object):
     def add_idx_line_col(self, key, data):
         # type: (Any, Any) -> None
         if self.data is None:
-            self.data = {}  # type: Dict[Any, Any]
+            self.data = {}
         self.data[key] = data
 
-
-class Anchor(object):
-    __slots__ = 'value', 'always_dump'
-    attrib = anchor_attrib
-
-    def __init__(self):
-        # type: () -> None
-        self.value = None
-        self.always_dump = False
+    def __repr__(self):
+        # type: () -> str
+        return _F('LineCol({line}, {col})', line=self.line, col=self.col)
 
 
-class Tag(object):
+class Tag:
     """store tag information for roundtripping"""
 
     __slots__ = ('value',)
@@ -188,7 +305,7 @@ class Tag(object):
         return '{0.__class__.__name__}({0.value!r})'.format(self)
 
 
-class CommentedBase(object):
+class CommentedBase:
     @property
     def ca(self):
         # type: () -> Any
@@ -234,12 +351,15 @@ class CommentedBase(object):
         from .error import CommentMark
         from .tokens import CommentToken
 
-        pre_comments = self._yaml_get_pre_comment()
+        pre_comments = self._yaml_clear_pre_comment()
         if comment[-1] == '\n':
             comment = comment[:-1]  # strip final newline if there
         start_mark = CommentMark(indent)
         for com in comment.split('\n'):
-            pre_comments.append(CommentToken('# ' + com + '\n', start_mark, None))
+            c = com.strip()
+            if len(c) > 0 and c[0] != '#':
+                com = '# ' + com
+            pre_comments.append(CommentToken(com + '\n', start_mark))
 
     def yaml_set_comment_before_after_key(
         self, key, before=None, indent=0, after=None, after_indent=None
@@ -254,7 +374,7 @@ class CommentedBase(object):
         def comment_token(s, mark):
             # type: (Any, Any) -> Any
             # handle empty lines as having no comment
-            return CommentToken(('# ' if s else "") + s + '\n', mark, None)
+            return CommentToken(('# ' if s else "") + s + '\n', mark)
 
         if after_indent is None:
             after_indent = indent + 2
@@ -297,7 +417,10 @@ class CommentedBase(object):
         from .error import CommentMark
 
         if column is None:
-            column = self._yaml_get_column(key)
+            try:
+                column = self._yaml_get_column(key)
+            except AttributeError:
+                column = 0
         if comment[0] != '#':
             comment = '# ' + comment
         if column is None:
@@ -305,7 +428,7 @@ class CommentedBase(object):
                 comment = ' ' + comment
                 column = 0
         start_mark = CommentMark(column)
-        ct = [CommentToken(comment, start_mark, None), None]
+        ct = [CommentToken(comment, start_mark), None]
         self._yaml_add_eol_comment(ct, key=key)
 
     @property
@@ -357,14 +480,14 @@ class CommentedBase(object):
         # type: (Any) -> None
         self.tag.value = value
 
-    def copy_attributes(self, t, deep=False):
-        # type: (Any, bool) -> None
+    def copy_attributes(self, t, memo=None):
+        # type: (Any, Any) -> None
         # fmt: off
         for a in [Comment.attrib, Format.attrib, LineCol.attrib, Anchor.attrib,
                   Tag.attrib, merge_attrib]:
             if hasattr(self, a):
-                if deep:
-                    setattr(t, a, copy.deepcopy(getattr(self, a)))
+                if memo is not None:
+                    setattr(t, a, copy.deepcopy(getattr(self, a, memo)))
                 else:
                     setattr(t, a, getattr(self, a))
         # fmt: on
@@ -382,8 +505,58 @@ class CommentedBase(object):
         raise NotImplementedError
 
 
-class CommentedSeq(list, CommentedBase):
-    __slots__ = (Comment.attrib,)
+class CommentedSeq(MutableSliceableSequence, list, CommentedBase):  # type: ignore
+    __slots__ = (Comment.attrib, '_lst')
+
+    def __init__(self, *args, **kw):
+        # type: (Any, Any) -> None
+        list.__init__(self, *args, **kw)
+
+    def __getsingleitem__(self, idx):
+        # type: (Any) -> Any
+        return list.__getitem__(self, idx)
+
+    def __setsingleitem__(self, idx, value):
+        # type: (Any, Any) -> None
+        # try to preserve the scalarstring type if setting an existing key to a new value
+        if idx < len(self):
+            if (
+                isinstance(value, str)
+                and not isinstance(value, ScalarString)
+                and isinstance(self[idx], ScalarString)
+            ):
+                value = type(self[idx])(value)
+        list.__setitem__(self, idx, value)
+
+    def __delsingleitem__(self, idx=None):
+        # type: (Any) -> Any
+        list.__delitem__(self, idx)
+        self.ca.items.pop(idx, None)  # might not be there -> default value
+        for list_index in sorted(self.ca.items):
+            if list_index < idx:
+                continue
+            self.ca.items[list_index - 1] = self.ca.items.pop(list_index)
+
+    def __len__(self):
+        # type: () -> int
+        return list.__len__(self)
+
+    def insert(self, idx, val):
+        # type: (Any, Any) -> None
+        """the comments after the insertion have to move forward"""
+        list.insert(self, idx, val)
+        for list_index in sorted(self.ca.items, reverse=True):
+            if list_index < idx:
+                break
+            self.ca.items[list_index + 1] = self.ca.items.pop(list_index)
+
+    def extend(self, val):
+        # type: (Any) -> None
+        list.extend(self, val)
+
+    def __eq__(self, other):
+        # type: (Any) -> bool
+        return list.__eq__(self, other)
 
     def _yaml_add_comment(self, comment, key=NoComment):
         # type: (Any, Optional[Any]) -> None
@@ -400,25 +573,6 @@ class CommentedSeq(list, CommentedBase):
         # type: (Any) -> Any
         return self.ca.items[key][0].start_mark.column
 
-    def insert(self, idx, val):
-        # type: (Any, Any) -> None
-        """the comments after the insertion have to move forward"""
-        list.insert(self, idx, val)
-        for list_index in sorted(self.ca.items, reverse=True):
-            if list_index < idx:
-                break
-            self.ca.items[list_index + 1] = self.ca.items.pop(list_index)
-
-    def pop(self, idx=None):
-        # type: (Any) -> Any
-        res = list.pop(self, idx)
-        self.ca.items.pop(idx, None)  # might not be there -> default value
-        for list_index in sorted(self.ca.items):
-            if list_index < idx:
-                continue
-            self.ca.items[list_index - 1] = self.ca.items.pop(list_index)
-        return res
-
     def _yaml_get_column(self, key):
         # type: (Any) -> Any
         column = None
@@ -441,6 +595,15 @@ class CommentedSeq(list, CommentedBase):
         return column
 
     def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            pre_comments = self.ca.comment[1]
+        return pre_comments
+
+    def _yaml_clear_pre_comment(self):
         # type: () -> Any
         pre_comments = []  # type: List[Any]
         if self.ca.comment is None:
@@ -454,24 +617,37 @@ class CommentedSeq(list, CommentedBase):
         res = self.__class__()
         memo[id(self)] = res
         for k in self:
-            res.append(copy.deepcopy(k))
-            self.copy_attributes(res, deep=True)
+            res.append(copy.deepcopy(k, memo))
+            self.copy_attributes(res, memo=memo)
         return res
 
-    def __setitem__(self, idx, value):
-        # type: (Any, Any) -> None
-        # try to preserve the scalarstring type if setting an existing key to a new value
-        if idx < len(self):
-            if (
-                isinstance(value, string_types)
-                and not isinstance(value, ScalarString)
-                and isinstance(self[idx], ScalarString)
-            ):
-                value = type(self[idx])(value)
-        list.__setitem__(self, idx, value)
+    def __add__(self, other):
+        # type: (Any) -> Any
+        return list.__add__(self, other)
+
+    def sort(self, key=None, reverse=False):  # type: ignore
+        # type: (Any, bool) -> None
+        if key is None:
+            tmp_lst = sorted(zip(self, range(len(self))), reverse=reverse)
+            list.__init__(self, [x[0] for x in tmp_lst])
+        else:
+            tmp_lst = sorted(
+                zip(map(key, list.__iter__(self)), range(len(self))), reverse=reverse
+            )
+            list.__init__(self, [list.__getitem__(self, x[1]) for x in tmp_lst])
+        itm = self.ca.items
+        self.ca._items = {}
+        for idx, x in enumerate(tmp_lst):
+            old_index = x[1]
+            if old_index in itm:
+                self.ca.items[idx] = itm[old_index]
+
+    def __repr__(self):
+        # type: () -> Any
+        return list.__repr__(self)
 
 
-class CommentedKeySeq(tuple, CommentedBase):
+class CommentedKeySeq(tuple, CommentedBase):  # type: ignore
     """This primarily exists to be able to roundtrip keys that are sequences"""
 
     def _yaml_add_comment(self, comment, key=NoComment):
@@ -511,6 +687,15 @@ class CommentedKeySeq(tuple, CommentedBase):
         return column
 
     def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            pre_comments = self.ca.comment[1]
+        return pre_comments
+
+    def _yaml_clear_pre_comment(self):
         # type: () -> Any
         pre_comments = []  # type: List[Any]
         if self.ca.comment is None:
@@ -530,25 +715,10 @@ class CommentedMapView(Sized):
     def __len__(self):
         # type: () -> int
         count = len(self._mapping)
-        done = []  # type: List[Any] # list of processed merge items, kept for masking
-        for merged in getattr(self._mapping, merge_attrib, []):
-            for x in merged[1]:
-                if self._mapping._unmerged_contains(x):
-                    continue
-                for y in done:
-                    if x in y:
-                        break
-                else:
-                    count += 1
-            done.append(merged[1])
         return count
 
-    def __repr__(self):
-        # type: () -> str
-        return '{0.__class__.__name__}({0._mapping!r})'.format(self)
 
-
-class CommentedMapKeysView(CommentedMapView, Set):
+class CommentedMapKeysView(CommentedMapView, Set):  # type: ignore
     __slots__ = ()
 
     @classmethod
@@ -562,11 +732,12 @@ class CommentedMapKeysView(CommentedMapView, Set):
 
     def __iter__(self):
         # type: () -> Any  # yield from self._mapping  # not in py27, pypy
+        # for x in self._mapping._keys():
         for x in self._mapping:
             yield x
 
 
-class CommentedMapItemsView(CommentedMapView, Set):
+class CommentedMapItemsView(CommentedMapView, Set):  # type: ignore
     __slots__ = ()
 
     @classmethod
@@ -602,12 +773,18 @@ class CommentedMapValuesView(CommentedMapView):
 
     def __iter__(self):
         # type: () -> Any
-        for key in self._mapping:
+        for key in self._mapping._keys():
             yield self._mapping[key]
 
 
-class CommentedMap(ordereddict, CommentedBase):
-    __slots__ = (Comment.attrib,)
+class CommentedMap(ordereddict, CommentedBase):  # type: ignore
+    __slots__ = (Comment.attrib, '_ok', '_ref')
+
+    def __init__(self, *args, **kw):
+        # type: (Any, Any) -> None
+        self._ok = set()  # type: MutableSet[Any]  #  own keys
+        self._ref = []  # type: List[CommentedMap]
+        ordereddict.__init__(self, *args, **kw)
 
     def _yaml_add_comment(self, comment, key=NoComment, value=NoComment):
         # type: (Any, Optional[Any], Optional[Any]) -> None
@@ -658,6 +835,15 @@ class CommentedMap(ordereddict, CommentedBase):
         return column
 
     def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            pre_comments = self.ca.comment[1]
+        return pre_comments
+
+    def _yaml_clear_pre_comment(self):
         # type: () -> Any
         pre_comments = []  # type: List[Any]
         if self.ca.comment is None:
@@ -666,14 +852,22 @@ class CommentedMap(ordereddict, CommentedBase):
             self.ca.comment[1] = pre_comments
         return pre_comments
 
-    def update(self, vals):
-        # type: (Any) -> None
+    def update(self, *vals, **kw):
+        # type: (Any, Any) -> None
         try:
-            ordereddict.update(self, vals)
+            ordereddict.update(self, *vals, **kw)
         except TypeError:
             # probably a dict that is used
-            for x in vals:
-                self[x] = vals[x]
+            for x in vals[0]:
+                self[x] = vals[0][x]
+        try:
+            self._ok.update(vals.keys())  # type: ignore
+        except AttributeError:
+            # assume one argument that is a list/tuple of two element lists/tuples
+            for x in vals[0]:
+                self._ok.add(x[0])
+        if kw:
+            self._ok.add(*kw.keys())
 
     def insert(self, pos, key, value, comment=None):
         # type: (Any, Any, Any, Optional[Any]) -> None
@@ -681,6 +875,7 @@ class CommentedMap(ordereddict, CommentedBase):
         attach comment if provided
         """
         ordereddict.insert(self, pos, key, value)
+        self._ok.add(key)
         if comment is not None:
             self.yaml_add_eol_comment(comment, key=key)
 
@@ -725,28 +920,23 @@ class CommentedMap(ordereddict, CommentedBase):
         # try to preserve the scalarstring type if setting an existing key to a new value
         if key in self:
             if (
-                isinstance(value, string_types)
+                isinstance(value, str)
                 and not isinstance(value, ScalarString)
                 and isinstance(self[key], ScalarString)
             ):
                 value = type(self[key])(value)
         ordereddict.__setitem__(self, key, value)
+        self._ok.add(key)
 
     def _unmerged_contains(self, key):
         # type: (Any) -> Any
-        if ordereddict.__contains__(self, key):
+        if key in self._ok:
             return True
         return None
 
     def __contains__(self, key):
         # type: (Any) -> bool
-        if ordereddict.__contains__(self, key):
-            return True
-        # this will only work once the mapping/dict is built to completion
-        for merged in getattr(self, merge_attrib, []):
-            if key in merged[1]:
-                return True
-        return False
+        return bool(ordereddict.__contains__(self, key))
 
     def get(self, key, default=None):
         # type: (Any, Any) -> Any
@@ -757,153 +947,68 @@ class CommentedMap(ordereddict, CommentedBase):
 
     def __repr__(self):
         # type: () -> Any
-        if not hasattr(self, merge_attrib):
-            return ordereddict.__repr__(self)
-        return 'ordereddict(' + repr(list(self._items())) + ')'
+        return ordereddict.__repr__(self).replace('CommentedMap', 'ordereddict')
 
     def non_merged_items(self):
         # type: () -> Any
         for x in ordereddict.__iter__(self):
-            yield x, ordereddict.__getitem__(self, x)
+            if x in self._ok:
+                yield x, ordereddict.__getitem__(self, x)
 
     def __delitem__(self, key):
         # type: (Any) -> None
-        found = True
-        for merged in getattr(self, merge_attrib, []):
-            try:
-                del merged[1][key]
-                found = True
-            except KeyError:
-                pass
-        try:
-            ordereddict.__delitem__(self, key)
-        except KeyError:
-            if not found:
-                raise
+        # for merged in getattr(self, merge_attrib, []):
+        #     if key in merged[1]:
+        #         value = merged[1][key]
+        #         break
+        # else:
+        #     # not found in merged in stuff
+        #     ordereddict.__delitem__(self, key)
+        #    for referer in self._ref:
+        #        referer.update_key_value(key)
+        #    return
+        #
+        # ordereddict.__setitem__(self, key, value)  # merge might have different value
+        # self._ok.discard(key)
+        self._ok.discard(key)
+        ordereddict.__delitem__(self, key)
+        for referer in self._ref:
+            referer.update_key_value(key)
 
     def __iter__(self):
         # type: () -> Any
         for x in ordereddict.__iter__(self):
             yield x
-        done = []  # type: List[Any]  # list of processed merge items, kept for masking
-        for merged in getattr(self, merge_attrib, []):
-            for x in merged[1]:
-                if ordereddict.__contains__(self, x):
-                    continue
-                for y in done:
-                    if x in y:
-                        break
-                else:
-                    yield x
-            done.append(merged[1])
 
     def _keys(self):
         # type: () -> Any
         for x in ordereddict.__iter__(self):
             yield x
-        done = []  # type: List[Any]  # list of processed merge items, kept for masking
-        for merged in getattr(self, merge_attrib, []):
-            for x in merged[1]:
-                if ordereddict.__contains__(self, x):
-                    continue
-                for y in done:
-                    if x in y:
-                        break
-                else:
-                    yield x
-            done.append(merged[1])
 
-    if PY2:
+    def __len__(self):
+        # type: () -> int
+        return int(ordereddict.__len__(self))
 
-        def keys(self):
-            # type: () -> Any
-            return list(self._keys())
+    def __eq__(self, other):
+        # type: (Any) -> bool
+        return bool(dict(self) == other)
 
-        def iterkeys(self):
-            # type: () -> Any
-            return self._keys()
-
-        def viewkeys(self):
-            # type: () -> Any
-            return CommentedMapKeysView(self)
-
-    else:
-
-        def keys(self):
-            # type: () -> Any
-            return CommentedMapKeysView(self)
-
-    def _values(self):
+    def keys(self):
         # type: () -> Any
-        for x in ordereddict.__iter__(self):
-            yield ordereddict.__getitem__(self, x)
-        done = []  # type: List[Any]  # list of processed merge items, kept for masking
-        for merged in getattr(self, merge_attrib, []):
-            for x in merged[1]:
-                if ordereddict.__contains__(self, x):
-                    continue
-                for y in done:
-                    if x in y:
-                        break
-                else:
-                    yield ordereddict.__getitem__(merged[1], x)
-            done.append(merged[1])
+        return CommentedMapKeysView(self)
 
-    if PY2:
-
-        def values(self):
-            # type: () -> Any
-            return list(self._values())
-
-        def itervalues(self):
-            # type: () -> Any
-            return self._values()
-
-        def viewvalues(self):
-            # type: () -> Any
-            return CommentedMapValuesView(self)
-
-    else:
-
-        def values(self):
-            # type: () -> Any
-            return CommentedMapValuesView(self)
+    def values(self):
+        # type: () -> Any
+        return CommentedMapValuesView(self)
 
     def _items(self):
         # type: () -> Any
         for x in ordereddict.__iter__(self):
             yield x, ordereddict.__getitem__(self, x)
-        done = []  # type: List[Any]  # list of processed merge items, kept for masking
-        for merged in getattr(self, merge_attrib, []):
-            for x, v in merged[1].items():
-                if ordereddict.__contains__(self, x):
-                    continue
-                for y in done:
-                    if x in y:
-                        break
-                else:
-                    yield x, v  # ordereddict.__getitem__(merged[1], x)
-            done.append(merged[1])
 
-    if PY2:
-
-        def items(self):
-            # type: () -> Any
-            return list(self._items())
-
-        def iteritems(self):
-            # type: () -> Any
-            return self._items()
-
-        def viewitems(self):
-            # type: () -> Any
-            return CommentedMapItemsView(self)
-
-    else:
-
-        def items(self):
-            # type: () -> Any
-            return CommentedMapItemsView(self)
+    def items(self):
+        # type: () -> Any
+        return CommentedMapItemsView(self)
 
     @property
     def merge(self):
@@ -912,25 +1017,151 @@ class CommentedMap(ordereddict, CommentedBase):
             setattr(self, merge_attrib, [])
         return getattr(self, merge_attrib)
 
+    def copy(self):
+        # type: () -> Any
+        x = type(self)()  # update doesn't work
+        for k, v in self._items():
+            x[k] = v
+        self.copy_attributes(x)
+        return x
+
+    def add_referent(self, cm):
+        # type: (Any) -> None
+        if cm not in self._ref:
+            self._ref.append(cm)
+
     def add_yaml_merge(self, value):
         # type: (Any) -> None
+        for v in value:
+            v[1].add_referent(self)
+            for k, v in v[1].items():
+                if ordereddict.__contains__(self, k):
+                    continue
+                ordereddict.__setitem__(self, k, v)
         self.merge.extend(value)
 
+    def update_key_value(self, key):
+        # type: (Any) -> None
+        if key in self._ok:
+            return
+        for v in self.merge:
+            if key in v[1]:
+                ordereddict.__setitem__(self, key, v[1][key])
+                return
+        ordereddict.__delitem__(self, key)
+
     def __deepcopy__(self, memo):
         # type: (Any) -> Any
         res = self.__class__()
         memo[id(self)] = res
         for k in self:
-            res[k] = copy.deepcopy(self[k])
-            self.copy_attributes(res, deep=True)
+            res[k] = copy.deepcopy(self[k], memo)
+        self.copy_attributes(res, memo=memo)
         return res
 
 
+# based on brownie mappings
+@classmethod  # type: ignore
+def raise_immutable(cls, *args, **kwargs):
+    # type: (Any, *Any, **Any) -> None
+    raise TypeError('{} objects are immutable'.format(cls.__name__))
+
+
+class CommentedKeyMap(CommentedBase, Mapping):  # type: ignore
+    __slots__ = Comment.attrib, '_od'
+    """This primarily exists to be able to roundtrip keys that are mappings"""
+
+    def __init__(self, *args, **kw):
+        # type: (Any, Any) -> None
+        if hasattr(self, '_od'):
+            raise_immutable(self)
+        try:
+            self._od = ordereddict(*args, **kw)
+        except TypeError:
+            raise
+
+    __delitem__ = __setitem__ = clear = pop = popitem = setdefault = update = raise_immutable
+
+    # need to implement __getitem__, __iter__ and __len__
+    def __getitem__(self, index):
+        # type: (Any) -> Any
+        return self._od[index]
+
+    def __iter__(self):
+        # type: () -> Iterator[Any]
+        for x in self._od.__iter__():
+            yield x
+
+    def __len__(self):
+        # type: () -> int
+        return len(self._od)
+
+    def __hash__(self):
+        # type: () -> Any
+        return hash(tuple(self.items()))
+
+    def __repr__(self):
+        # type: () -> Any
+        if not hasattr(self, merge_attrib):
+            return self._od.__repr__()
+        return 'ordereddict(' + repr(list(self._od.items())) + ')'
+
+    @classmethod
+    def fromkeys(keys, v=None):
+        # type: (Any, Any) -> Any
+        return CommentedKeyMap(dict.fromkeys(keys, v))
+
+    def _yaml_add_comment(self, comment, key=NoComment):
+        # type: (Any, Optional[Any]) -> None
+        if key is not NoComment:
+            self.yaml_key_comment_extend(key, comment)
+        else:
+            self.ca.comment = comment
+
+    def _yaml_add_eol_comment(self, comment, key):
+        # type: (Any, Any) -> None
+        self._yaml_add_comment(comment, key=key)
+
+    def _yaml_get_columnX(self, key):
+        # type: (Any) -> Any
+        return self.ca.items[key][0].start_mark.column
+
+    def _yaml_get_column(self, key):
+        # type: (Any) -> Any
+        column = None
+        sel_idx = None
+        pre, post = key - 1, key + 1
+        if pre in self.ca.items:
+            sel_idx = pre
+        elif post in self.ca.items:
+            sel_idx = post
+        else:
+            # self.ca.items is not ordered
+            for row_idx, _k1 in enumerate(self):
+                if row_idx >= key:
+                    break
+                if row_idx not in self.ca.items:
+                    continue
+                sel_idx = row_idx
+        if sel_idx is not None:
+            column = self._yaml_get_columnX(sel_idx)
+        return column
+
+    def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            self.ca.comment[1] = pre_comments
+        return pre_comments
+
+
 class CommentedOrderedMap(CommentedMap):
     __slots__ = (Comment.attrib,)
 
 
-class CommentedSet(MutableSet, CommentedMap):
+class CommentedSet(MutableSet, CommentedBase):  # type: ignore  # NOQA
     __slots__ = Comment.attrib, 'odict'
 
     def __init__(self, values=None):
@@ -940,6 +1171,22 @@ class CommentedSet(MutableSet, CommentedMap):
         if values is not None:
             self |= values  # type: ignore
 
+    def _yaml_add_comment(self, comment, key=NoComment, value=NoComment):
+        # type: (Any, Optional[Any], Optional[Any]) -> None
+        """values is set to key to indicate a value attachment of comment"""
+        if key is not NoComment:
+            self.yaml_key_comment_extend(key, comment)
+            return
+        if value is not NoComment:
+            self.yaml_value_comment_extend(value, comment)
+        else:
+            self.ca.comment = comment
+
+    def _yaml_add_eol_comment(self, comment, key):
+        # type: (Any, Any) -> None
+        """add on the value line, with value specified by the key"""
+        self._yaml_add_comment(comment, value=key)
+
     def add(self, value):
         # type: (Any) -> None
         """Add an element."""
@@ -970,10 +1217,12 @@ class CommentedSet(MutableSet, CommentedMap):
 
 class TaggedScalar(CommentedBase):
     # the value and style attributes are set during roundtrip construction
-    def __init__(self):
-        # type: () -> None
-        self.value = None
-        self.style = None
+    def __init__(self, value=None, style=None, tag=None):
+        # type: (Any, Any, Any) -> None
+        self.value = value
+        self.style = style
+        if tag is not None:
+            self.yaml_set_tag(tag)
 
     def __str__(self):
         # type: () -> Any
@@ -988,14 +1237,14 @@ def dump_comments(d, name="", sep='.', out=sys.stdout):
     """
     if isinstance(d, dict) and hasattr(d, 'ca'):
         if name:
-            print(name)
-        print(d.ca, file=out)  # type: ignore
+            out.write('{} {}\n'.format(name, type(d)))
+        out.write('{!r}\n'.format(d.ca))  # type: ignore
         for k in d:
-            dump_comments(d[k], name=(name + sep + k) if name else k, sep=sep, out=out)
+            dump_comments(d[k], name=(name + sep + str(k)) if name else k, sep=sep, out=out)
     elif isinstance(d, list) and hasattr(d, 'ca'):
         if name:
-            print(name)
-        print(d.ca, file=out)  # type: ignore
+            out.write('{} {}\n'.format(name, type(d)))
+        out.write('{!r}\n'.format(d.ca))  # type: ignore
         for idx, k in enumerate(d):
             dump_comments(
                 k, name=(name + sep + str(idx)) if name else str(idx), sep=sep, out=out
