@@ -1,17 +1,19 @@
 # coding: utf-8
 
-from __future__ import print_function
-
 # partially from package six by Benjamin Peterson
 
 import sys
 import os
-import types
+import io
+import traceback
 from abc import abstractmethod
+import collections.abc
+
 
 # fmt: off
 if False:  # MYPY
-    from typing import Any, Dict, Optional, List, Union, BinaryIO, IO, Text, Tuple, Optional  # NOQA
+    from typing import Any, Dict, Optional, List, Union, BinaryIO, IO, Text, Tuple  # NOQA
+    from typing import Optional  # NOQA
 # fmt: on
 
 _DEFAULT_YAML_VERSION = (1, 2)
@@ -47,85 +49,28 @@ PY2 = sys.version_info[0] == 2
 PY3 = sys.version_info[0] == 3
 
 
-if PY3:
-
-    def utf8(s):
-        # type: (str) -> str
-        return s
-
-    def to_str(s):
-        # type: (str) -> str
-        return s
-
-    def to_unicode(s):
-        # type: (str) -> str
-        return s
+# replace with f-strings when 3.5 support is dropped
+# ft = '42'
+# assert _F('abc {ft!r}', ft=ft) == 'abc %r' % ft
+# 'abc %r' % ft -> _F('abc {ft!r}' -> f'abc {ft!r}'
+def _F(s, *superfluous, **kw):
+    if superfluous:
+        raise TypeError
+    return s.format(**kw)
 
 
-else:
-
-    def utf8(s):
-        # type: (unicode) -> str
-        return s.encode('utf-8')
-
-    def to_str(s):
-        # type: (str) -> str
-        return str(s)
-
-    def to_unicode(s):
-        # type: (str) -> unicode
-        return unicode(s)  # NOQA
-
-
-if PY3:
-    string_types = str
-    integer_types = int
-    class_types = type
-    text_type = str
-    binary_type = bytes
-
-    MAXSIZE = sys.maxsize
-    unichr = chr
-    import io
-
-    StringIO = io.StringIO
-    BytesIO = io.BytesIO
-    # have unlimited precision
-    no_limit_int = int
-    from collections.abc import Hashable, MutableSequence  # NOQA
-
-else:
-    string_types = basestring  # NOQA
-    integer_types = (int, long)  # NOQA
-    class_types = (type, types.ClassType)
-    text_type = unicode  # NOQA
-    binary_type = str
-
-    # to allow importing
-    unichr = unichr  # type: ignore
-    from StringIO import StringIO as _StringIO
-
-    StringIO = _StringIO
-    import cStringIO
-
-    BytesIO = cStringIO.StringIO
-    # have unlimited precision
-    no_limit_int = long  # NOQA not available on Python 3
-    from collections import Hashable, MutableSequence  # NOQA
+StringIO = io.StringIO
+BytesIO = io.BytesIO
 
 if False:  # MYPY
     # StreamType = Union[BinaryIO, IO[str], IO[unicode],  StringIO]
-    StreamType = Union[BinaryIO, IO[str], StringIO]
+    # StreamType = Union[BinaryIO, IO[str], StringIO]  # type: ignore
+    StreamType = Any
 
-    StreamTextType = Union[Text, StreamType]
+    StreamTextType = StreamType  # Union[Text, StreamType]
     VersionType = Union[List[int], str, Tuple[int, int]]
 
-if PY3:
-    builtins_module = 'builtins'
-else:
-    builtins_module = '__builtin__'
-
-UNICODE_SIZE = 4 if sys.maxunicode > 65535 else 2
+builtins_module = 'builtins'
 
 
 def with_metaclass(meta, *bases):
@@ -150,7 +95,7 @@ if 'RUAMELDEBUG' in os.environ:
 
 if bool(_debug):
 
-    class ObjectCounter(object):
+    class ObjectCounter:
         def __init__(self):
             # type: () -> None
             self.map = {}  # type: Dict[Any, Any]
@@ -162,7 +107,7 @@ if bool(_debug):
         def dump(self):
             # type: () -> None
             for k in sorted(self.map):
-                print(k, '->', self.map[k])
+                sys.stdout.write('{} -> {}'.format(k, self.map[k]))
 
     object_counter = ObjectCounter()
 
@@ -183,37 +128,76 @@ def dbg(val=None):
     return _debug & val
 
 
-def nprint(*args, **kw):
-    # type: (Any, Any) -> None
-    if bool(dbg):
-        print(*args, **kw)
+class Nprint:
+    def __init__(self, file_name=None):
+        # type: (Any) -> None
+        self._max_print = None  # type: Any
+        self._count = None  # type: Any
+        self._file_name = file_name
 
+    def __call__(self, *args, **kw):
+        # type: (Any, Any) -> None
+        if not bool(_debug):
+            return
+        out = sys.stdout if self._file_name is None else open(self._file_name, 'a')
+        dbgprint = print  # to fool checking for print statements by dv utility
+        kw1 = kw.copy()
+        kw1['file'] = out
+        dbgprint(*args, **kw1)
+        out.flush()
+        if self._max_print is not None:
+            if self._count is None:
+                self._count = self._max_print
+            self._count -= 1
+            if self._count == 0:
+                dbgprint('forced exit\n')
+                traceback.print_stack()
+                out.flush()
+                sys.exit(0)
+        if self._file_name:
+            out.close()
+
+    def set_max_print(self, i):
+        # type: (int) -> None
+        self._max_print = i
+        self._count = None
+
+    def fp(self, mode='a'):
+        out = sys.stdout if self._file_name is None else open(self._file_name, mode)
+        return out
+
+
+nprint = Nprint()
+nprintf = Nprint('/var/tmp/ruamel.yaml.log')
 
 # char checkers following production rules
 
 
 def check_namespace_char(ch):
     # type: (Any) -> bool
-    if u'\x21' <= ch <= u'\x7E':  # ! to ~
+    if '\x21' <= ch <= '\x7E':  # ! to ~
         return True
-    if u'\xA0' <= ch <= u'\xD7FF':
+    if '\xA0' <= ch <= '\uD7FF':
         return True
-    if (u'\xE000' <= ch <= u'\xFFFD') and ch != u'\xFEFF':  # excl. byte order mark
+    if ('\uE000' <= ch <= '\uFFFD') and ch != '\uFEFF':  # excl. byte order mark
         return True
-    if u'\x10000' <= ch <= u'\x10FFFF':
+    if '\U00010000' <= ch <= '\U0010FFFF':
         return True
     return False
 
 
 def check_anchorname_char(ch):
     # type: (Any) -> bool
-    if ch in u',[]{}':
+    if ch in ',[]{}':
         return False
     return check_namespace_char(ch)
 
 
 def version_tnf(t1, t2=None):
     # type: (Any, Any) -> Any
+    """
+    return True if ruamel.yaml version_info < t1, None if t2 is specified and bigger else False
+    """
     from ruamel.yaml import version_info  # NOQA
 
     if version_info < t1:
@@ -223,7 +207,7 @@ def version_tnf(t1, t2=None):
     return False
 
 
-class MutableSliceableSequence(MutableSequence):
+class MutableSliceableSequence(collections.abc.MutableSequence):  # type: ignore
     __slots__ = ()
 
     def __getitem__(self, index):
@@ -237,7 +221,7 @@ class MutableSliceableSequence(MutableSequence):
         if not isinstance(index, slice):
             return self.__setsingleitem__(index, value)
         assert iter(value)
-        print(index.start, index.stop, index.step, index.indices(len(self)))
+        # nprint(index.start, index.stop, index.step, index.indices(len(self)))
         if index.step is None:
             del self[index.start : index.stop]
             for elem in reversed(value):
@@ -263,7 +247,7 @@ class MutableSliceableSequence(MutableSequence):
         # type: (Any) -> None
         if not isinstance(index, slice):
             return self.__delsingleitem__(index)
-        print(index.start, index.stop, index.step, index.indices(len(self)))
+        # nprint(index.start, index.stop, index.step, index.indices(len(self)))
         for i in reversed(range(*index.indices(len(self)))):
             del self[i]
 
