@@ -1,20 +1,21 @@
 # coding: utf-8
 
-from __future__ import absolute_import
-from __future__ import print_function
-
 import warnings
 
 from ruamel.yaml.error import MarkedYAMLError, ReusedAnchorWarning
-from ruamel.yaml.compat import utf8
+from ruamel.yaml.compat import _F, nprint, nprintf  # NOQA
 
 from ruamel.yaml.events import (
-    StreamStartEvent, StreamEndEvent, MappingStartEvent, MappingEndEvent,
-    SequenceStartEvent, SequenceEndEvent, AliasEvent, ScalarEvent,
-)
-from ruamel.yaml.nodes import (
-    MappingNode, ScalarNode, SequenceNode,
+    StreamStartEvent,
+    StreamEndEvent,
+    MappingStartEvent,
+    MappingEndEvent,
+    SequenceStartEvent,
+    SequenceEndEvent,
+    AliasEvent,
+    ScalarEvent,
 )
+from ruamel.yaml.nodes import MappingNode, ScalarNode, SequenceNode
 
 if False:  # MYPY
     from typing import Any, Dict, Optional, List  # NOQA
@@ -26,7 +27,7 @@ class ComposerError(MarkedYAMLError):
     pass
 
 
-class Composer(object):
+class Composer:
     def __init__(self, loader=None):
         # type: (Any) -> None
         self.loader = loader
@@ -38,16 +39,16 @@ class Composer(object):
     def parser(self):
         # type: () -> Any
         if hasattr(self.loader, 'typ'):
-            self.loader.parser  # type: ignore
-        return self.loader._parser  # type: ignore
+            self.loader.parser
+        return self.loader._parser
 
     @property
     def resolver(self):
         # type: () -> Any
         # assert self.loader._resolver is not None
         if hasattr(self.loader, 'typ'):
-            self.loader.resolver  # type: ignore
-        return self.loader._resolver  # type: ignore
+            self.loader.resolver
+        return self.loader._resolver
 
     def check_node(self):
         # type: () -> Any
@@ -78,9 +79,11 @@ class Composer(object):
         if not self.parser.check_event(StreamEndEvent):
             event = self.parser.get_event()
             raise ComposerError(
-                "expected a single document in the stream",
-                document.start_mark, "but found another document",
-                event.start_mark)
+                'expected a single document in the stream',
+                document.start_mark,
+                'but found another document',
+                event.start_mark,
+            )
 
         # Drop the STREAM-END event.
         self.parser.get_event()
@@ -101,6 +104,9 @@ class Composer(object):
         self.anchors = {}
         return node
 
+    def return_alias(self, a):
+        return a
+
     def compose_node(self, parent, index):
         # type: (Any, Any) -> Any
         if self.parser.check_event(AliasEvent):
@@ -108,20 +114,24 @@ class Composer(object):
             alias = event.anchor
             if alias not in self.anchors:
                 raise ComposerError(
-                    None, None, "found undefined alias %r"
-                    % utf8(alias), event.start_mark)
-            return self.anchors[alias]
+                    None,
+                    None,
+                    _F('found undefined alias {alias!r}', alias=alias),
+                    event.start_mark,
+                )
+            return self.return_alias(self.anchors[alias])
         event = self.parser.peek_event()
         anchor = event.anchor
         if anchor is not None:  # have an anchor
             if anchor in self.anchors:
                 # raise ComposerError(
                 #     "found duplicate anchor %r; first occurrence"
-                #     % utf8(anchor), self.anchors[anchor].start_mark,
+                #     % (anchor), self.anchors[anchor].start_mark,
                 #     "second occurrence", event.start_mark)
-                ws = "\nfound duplicate anchor {!r}\nfirst occurrence {}\nsecond occurrence "\
-                     "{}".format(
-                         (anchor), self.anchors[anchor].start_mark, event.start_mark)
+                ws = (
+                    '\nfound duplicate anchor {!r}\nfirst occurrence {}\nsecond occurrence '
+                    '{}'.format((anchor), self.anchors[anchor].start_mark, event.start_mark)
+                )
                 warnings.warn(ws, ReusedAnchorWarning)
         self.resolver.descend_resolver(parent, index)
         if self.parser.check_event(ScalarEvent):
@@ -137,11 +147,17 @@ class Composer(object):
         # type: (Any) -> Any
         event = self.parser.get_event()
         tag = event.tag
-        if tag is None or tag == u'!':
+        if tag is None or tag == '!':
             tag = self.resolver.resolve(ScalarNode, event.value, event.implicit)
-        node = ScalarNode(tag, event.value,
-                          event.start_mark, event.end_mark, style=event.style,
-                          comment=event.comment)
+        node = ScalarNode(
+            tag,
+            event.value,
+            event.start_mark,
+            event.end_mark,
+            style=event.style,
+            comment=event.comment,
+            anchor=anchor,
+        )
         if anchor is not None:
             self.anchors[anchor] = node
         return node
@@ -150,12 +166,17 @@ class Composer(object):
         # type: (Any) -> Any
         start_event = self.parser.get_event()
         tag = start_event.tag
-        if tag is None or tag == u'!':
+        if tag is None or tag == '!':
             tag = self.resolver.resolve(SequenceNode, None, start_event.implicit)
-        node = SequenceNode(tag, [],
-                            start_event.start_mark, None,
-                            flow_style=start_event.flow_style,
-                            comment=start_event.comment, anchor=anchor)
+        node = SequenceNode(
+            tag,
+            [],
+            start_event.start_mark,
+            None,
+            flow_style=start_event.flow_style,
+            comment=start_event.comment,
+            anchor=anchor,
+        )
         if anchor is not None:
             self.anchors[anchor] = node
         index = 0
@@ -165,8 +186,10 @@ class Composer(object):
         end_event = self.parser.get_event()
         if node.flow_style is True and end_event.comment is not None:
             if node.comment is not None:
-                print('Warning: unexpected end_event commment in sequence '
-                      'node {}'.format(node.flow_style))
+                nprint(
+                    'Warning: unexpected end_event commment in sequence '
+                    'node {}'.format(node.flow_style)
+                )
             node.comment = end_event.comment
         node.end_mark = end_event.end_mark
         self.check_end_doc_comment(end_event, node)
@@ -176,12 +199,17 @@ class Composer(object):
         # type: (Any) -> Any
         start_event = self.parser.get_event()
         tag = start_event.tag
-        if tag is None or tag == u'!':
+        if tag is None or tag == '!':
             tag = self.resolver.resolve(MappingNode, None, start_event.implicit)
-        node = MappingNode(tag, [],
-                           start_event.start_mark, None,
-                           flow_style=start_event.flow_style,
-                           comment=start_event.comment, anchor=anchor)
+        node = MappingNode(
+            tag,
+            [],
+            start_event.start_mark,
+            None,
+            flow_style=start_event.flow_style,
+            comment=start_event.comment,
+            anchor=anchor,
+        )
         if anchor is not None:
             self.anchors[anchor] = node
         while not self.parser.check_event(MappingEndEvent):
