@@ -1,24 +1,24 @@
 # coding: utf-8
 
-from __future__ import print_function
+import sys
 
-from .compat import string_types
+from ruamel.yaml.compat import _F
 
 if False:  # MYPY
     from typing import Dict, Any, Text  # NOQA
 
 
-class Node(object):
+class Node:
     __slots__ = 'tag', 'value', 'start_mark', 'end_mark', 'comment', 'anchor'
 
-    def __init__(self, tag, value, start_mark, end_mark, comment=None):
-        # type: (Any, Any, Any, Any, Any) -> None
+    def __init__(self, tag, value, start_mark, end_mark, comment=None, anchor=None):
+        # type: (Any, Any, Any, Any, Any, Any) -> None
         self.tag = tag
         self.value = value
         self.start_mark = start_mark
         self.end_mark = end_mark
         self.comment = comment
-        self.anchor = None
+        self.anchor = anchor
 
     def __repr__(self):
         # type: () -> str
@@ -29,29 +29,36 @@ class Node(object):
         #     elif len(value) == 1:
         #         value = '<1 item>'
         #     else:
-        #         value = '<%d items>' % len(value)
+        #         value = f'<{len(value)} items>'
         # else:
         #     if len(value) > 75:
-        #         value = repr(value[:70]+u' ... ')
+        #         value = repr(value[:70]+' ... ')
         #     else:
         #         value = repr(value)
         value = repr(value)
-        return '%s(tag=%r, value=%s)' % (self.__class__.__name__, self.tag, value)
+        return _F(
+            '{class_name!s}(tag={self_tag!r}, value={value!s})',
+            class_name=self.__class__.__name__,
+            self_tag=self.tag,
+            value=value,
+        )
 
     def dump(self, indent=0):
         # type: (int) -> None
-        if isinstance(self.value, string_types):
-            print(
-                '{}{}(tag={!r}, value={!r})'.format(
+        if isinstance(self.value, str):
+            sys.stdout.write(
+                '{}{}(tag={!r}, value={!r})\n'.format(
                     '  ' * indent, self.__class__.__name__, self.tag, self.value
                 )
             )
             if self.comment:
-                print('    {}comment: {})'.format('  ' * indent, self.comment))
+                sys.stdout.write('    {}comment: {})\n'.format('  ' * indent, self.comment))
             return
-        print('{}{}(tag={!r})'.format('  ' * indent, self.__class__.__name__, self.tag))
+        sys.stdout.write(
+            '{}{}(tag={!r})\n'.format('  ' * indent, self.__class__.__name__, self.tag)
+        )
         if self.comment:
-            print('    {}comment: {})'.format('  ' * indent, self.comment))
+            sys.stdout.write('    {}comment: {})\n'.format('  ' * indent, self.comment))
         for v in self.value:
             if isinstance(v, tuple):
                 for v1 in v:
@@ -59,7 +66,7 @@ class Node(object):
             elif isinstance(v, Node):
                 v.dump(indent + 1)
             else:
-                print('Node value type?', type(v))
+                sys.stdout.write('Node value type? {}\n'.format(type(v)))
 
 
 class ScalarNode(Node):
@@ -75,14 +82,16 @@ class ScalarNode(Node):
     __slots__ = ('style',)
     id = 'scalar'
 
-    def __init__(self, tag, value, start_mark=None, end_mark=None, style=None, comment=None):
-        # type: (Any, Any, Any, Any, Any, Any) -> None
-        Node.__init__(self, tag, value, start_mark, end_mark, comment=comment)
+    def __init__(
+        self, tag, value, start_mark=None, end_mark=None, style=None, comment=None, anchor=None
+    ):
+        # type: (Any, Any, Any, Any, Any, Any, Any) -> None
+        Node.__init__(self, tag, value, start_mark, end_mark, comment=comment, anchor=anchor)
         self.style = style
 
 
 class CollectionNode(Node):
-    __slots__ = 'flow_style', 'anchor'
+    __slots__ = ('flow_style',)
 
     def __init__(
         self,
