@@ -47,7 +47,7 @@ class RepresenterError(YAMLError):
     pass
 
 
-class BaseRepresenter(object):
+class BaseRepresenter:
 
     yaml_representers = {}  # type: Dict[Any, Any]
     yaml_multi_representers = {}  # type: Dict[Any, Any]
@@ -837,13 +837,17 @@ class RoundTripRepresenter(SafeRepresenter):
                 for ct in node.comment[1]:
                     ct.reset()
             item_comments = comment.items
-            for v in item_comments.values():
-                if v and v[1]:
-                    for ct in v[1]:
-                        ct.reset()
-            try:
-                node.comment.append(comment.end)
-            except AttributeError:
+            if self.dumper.comment_handling is None:
+                for v in item_comments.values():
+                    if v and v[1]:
+                        for ct in v[1]:
+                            ct.reset()
+                try:
+                    node.comment.append(comment.end)
+                except AttributeError:
+                    pass
+            else:
+                # NEWCMNT
                 pass
         except AttributeError:
             item_comments = {}
@@ -863,7 +867,9 @@ class RoundTripRepresenter(SafeRepresenter):
             node_value = self.represent_data(item_value)
             item_comment = item_comments.get(item_key)
             if item_comment:
-                assert getattr(node_key, 'comment', None) is None
+                # assert getattr(node_key, 'comment', None) is None
+                # issue 351 did throw this because the comment from the list item was
+                # moved to the dict
                 node_key.comment = item_comment[:2]
                 nvc = getattr(node_value, 'comment', None)
                 if nvc is not None:  # end comment already there
