@@ -1,7 +1,5 @@
 # coding: utf-8
 
-from __future__ import print_function, absolute_import, division, unicode_literals
-
 # Scanner produces tokens of the following types:
 # STREAM-START
 # STREAM-END
@@ -30,9 +28,10 @@ from __future__ import print_function, absolute_import, division, unicode_litera
 # Read comments in the Scanner code for more details.
 #
 
-from ruamel.yaml.error import MarkedYAMLError
+import inspect
+from ruamel.yaml.error import MarkedYAMLError, CommentMark  # NOQA
 from ruamel.yaml.tokens import *  # NOQA
-from ruamel.yaml.compat import utf8, unichr, PY3, check_anchorname_char, nprint  # NOQA
+from ruamel.yaml.compat import _F, check_anchorname_char, nprint, nprintf  # NOQA
 
 if False:  # MYPY
     from typing import Any, Dict, Optional, List, Union, Text  # NOQA
@@ -46,11 +45,16 @@ _THE_END_SPACE_TAB = ' \n\0\t\r\x85\u2028\u2029'
 _SPACE_TAB = ' \t'
 
 
+def xprintf(*args, **kw):
+    return nprintf(*args, **kw)
+    pass
+
+
 class ScannerError(MarkedYAMLError):
     pass
 
 
-class SimpleKey(object):
+class SimpleKey:
     # See below simple keys treatment.
 
     def __init__(self, token_number, required, index, line, column, mark):
@@ -63,7 +67,7 @@ class SimpleKey(object):
         self.mark = mark
 
 
-class Scanner(object):
+class Scanner:
     def __init__(self, loader=None):
         # type: (Any) -> None
         """Initialize the scanner."""
@@ -168,7 +172,7 @@ class Scanner(object):
         # Check if the next token is one of the given types.
         while self.need_more_tokens():
             self.fetch_more_tokens()
-        if bool(self.tokens):
+        if len(self.tokens) > 0:
             if not choices:
                 return True
             for choice in choices:
@@ -181,7 +185,7 @@ class Scanner(object):
         # Return the next token, but do not delete if from the queue.
         while self.need_more_tokens():
             self.fetch_more_tokens()
-        if bool(self.tokens):
+        if len(self.tokens) > 0:
             return self.tokens[0]
 
     def get_token(self):
@@ -189,7 +193,7 @@ class Scanner(object):
         # Return the next token.
         while self.need_more_tokens():
             self.fetch_more_tokens()
-        if bool(self.tokens):
+        if len(self.tokens) > 0:
             self.tokens_taken += 1
             return self.tokens.pop(0)
 
@@ -199,7 +203,7 @@ class Scanner(object):
         # type: () -> bool
         if self.done:
             return False
-        if not self.tokens:
+        if len(self.tokens) == 0:
             return True
         # The current token may be a potential simple key, so we
         # need to look further.
@@ -245,7 +249,7 @@ class Scanner(object):
             return self.fetch_document_end()
 
         # TODO: support for BOM within a stream.
-        # if ch == u'\uFEFF':
+        # if ch == '\uFEFF':
         #     return self.fetch_bom()    <-- issue BOMToken
 
         # Note: the order of the following checks is NOT significant.
@@ -318,7 +322,7 @@ class Scanner(object):
         raise ScannerError(
             'while scanning for the next token',
             None,
-            'found character %r that cannot start any token' % utf8(ch),
+            _F('found character {ch!r} that cannot start any token', ch=ch),
             self.reader.get_mark(),
         )
 
@@ -917,7 +921,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                'expected alphabetic or numeric character, but found %r' % utf8(ch),
+                _F('expected alphabetic or numeric character, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         value = self.reader.prefix(length)
@@ -927,7 +931,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                'expected alphabetic or numeric character, but found %r' % utf8(ch),
+                _F('expected alphabetic or numeric character, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         return value
@@ -944,7 +948,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                "expected a digit or '.', but found %r" % utf8(srp()),
+                _F("expected a digit or '.', but found {srp_call!r}", srp_call=srp()),
                 self.reader.get_mark(),
             )
         srf()
@@ -953,7 +957,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                "expected a digit or ' ', but found %r" % utf8(srp()),
+                _F("expected a digit or '.', but found {srp_call!r}", srp_call=srp()),
                 self.reader.get_mark(),
             )
         self.yaml_version = (major, minor)
@@ -969,7 +973,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                'expected a digit, but found %r' % utf8(ch),
+                _F('expected a digit, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         length = 0
@@ -1001,7 +1005,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                "expected ' ', but found %r" % utf8(ch),
+                _F("expected ' ', but found {ch!r}", ch=ch),
                 self.reader.get_mark(),
             )
         return value
@@ -1015,7 +1019,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                "expected ' ', but found %r" % utf8(ch),
+                _F("expected ' ', but found {ch!r}", ch=ch),
                 self.reader.get_mark(),
             )
         return value
@@ -1035,7 +1039,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a directive',
                 start_mark,
-                'expected a comment or a line break, but found %r' % utf8(ch),
+                _F('expected a comment or a line break, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         self.scan_line_break()
@@ -1060,16 +1064,16 @@ class Scanner(object):
         self.reader.forward()
         length = 0
         ch = srp(length)
-        # while u'0' <= ch <= u'9' or u'A' <= ch <= u'Z' or u'a' <= ch <= u'z' \
-        #         or ch in u'-_':
+        # while '0' <= ch <= '9' or 'A' <= ch <= 'Z' or 'a' <= ch <= 'z' \
+        #         or ch in '-_':
         while check_anchorname_char(ch):
             length += 1
             ch = srp(length)
         if not length:
             raise ScannerError(
-                'while scanning an %s' % (name,),
+                _F('while scanning an {name!s}', name=name),
                 start_mark,
-                'expected alphabetic or numeric character, but found %r' % utf8(ch),
+                _F('expected alphabetic or numeric character, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         value = self.reader.prefix(length)
@@ -1079,9 +1083,9 @@ class Scanner(object):
         # assert ch1 == ch
         if ch not in '\0 \t\r\n\x85\u2028\u2029?:,[]{}%@`':
             raise ScannerError(
-                'while scanning an %s' % (name,),
+                _F('while scanning an {name!s}', name=name),
                 start_mark,
-                'expected alphabetic or numeric character, but found %r' % utf8(ch),
+                _F('expected alphabetic or numeric character, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         end_mark = self.reader.get_mark()
@@ -1101,7 +1105,7 @@ class Scanner(object):
                 raise ScannerError(
                     'while parsing a tag',
                     start_mark,
-                    "expected '>', but found %r" % utf8(srp()),
+                    _F("expected '>', but found {srp_call!r}", srp_call=srp()),
                     self.reader.get_mark(),
                 )
             self.reader.forward()
@@ -1130,7 +1134,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a tag',
                 start_mark,
-                "expected ' ', but found %r" % utf8(ch),
+                _F("expected ' ', but found {ch!r}", ch=ch),
                 self.reader.get_mark(),
             )
         value = (handle, suffix)
@@ -1209,10 +1213,10 @@ class Scanner(object):
                 # This is Clark Evans's interpretation (also in the spec
                 # examples):
                 #
-                # if folded and line_break == u'\n':
+                # if folded and line_break == '\n':
                 #     if not breaks:
                 #         if srp() not in ' \t':
-                #             chunks.append(u' ')
+                #             chunks.append(' ')
                 #         else:
                 #             chunks.append(line_break)
                 # else:
@@ -1232,21 +1236,33 @@ class Scanner(object):
 
         # We are done.
         token = ScalarToken("".join(chunks), False, start_mark, end_mark, style)
-        if block_scalar_comment is not None:
-            token.add_pre_comments([block_scalar_comment])
+        if self.loader is not None:
+            comment_handler = getattr(self.loader, 'comment_handling', False)
+            if comment_handler is None:
+                if block_scalar_comment is not None:
+                    token.add_pre_comments([block_scalar_comment])
         if len(trailing) > 0:
-            # nprint('trailing 1', trailing)  # XXXXX
             # Eat whitespaces and comments until we reach the next token.
+            if self.loader is not None:
+                comment_handler = getattr(self.loader, 'comment_handling', None)
+                if comment_handler is not None:
+                    line = end_mark.line - len(trailing)
+                    for x in trailing:
+                        assert x[-1] == '\n'
+                        self.comments.add_blank_line(x, 0, line)
+                        line += 1
             comment = self.scan_to_next_token()
             while comment:
                 trailing.append(' ' * comment[1].column + comment[0])
                 comment = self.scan_to_next_token()
-
-            # Keep track of the trailing whitespace and following comments
-            # as a comment token, if isn't all included in the actual value.
-            comment_end_mark = self.reader.get_mark()
-            comment = CommentToken("".join(trailing), end_mark, comment_end_mark)
-            token.add_post_comment(comment)
+            if self.loader is not None:
+                comment_handler = getattr(self.loader, 'comment_handling', False)
+                if comment_handler is None:
+                    # Keep track of the trailing whitespace and following comments
+                    # as a comment token, if isn't all included in the actual value.
+                    comment_end_mark = self.reader.get_mark()
+                    comment = CommentToken("".join(trailing), end_mark, comment_end_mark)
+                    token.add_post_comment(comment)
         return token
 
     def scan_block_scalar_indicators(self, start_mark):
@@ -1295,7 +1311,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a block scalar',
                 start_mark,
-                'expected chomping or indentation indicators, but found %r' % utf8(ch),
+                _F('expected chomping or indentation indicators, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         return chomping, increment
@@ -1320,7 +1336,7 @@ class Scanner(object):
             raise ScannerError(
                 'while scanning a block scalar',
                 start_mark,
-                'expected a comment or a line break, but found %r' % utf8(ch),
+                _F('expected a comment or a line break, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         self.scan_line_break()
@@ -1442,12 +1458,16 @@ class Scanner(object):
                             raise ScannerError(
                                 'while scanning a double-quoted scalar',
                                 start_mark,
-                                'expected escape sequence of %d hexdecimal '
-                                'numbers, but found %r' % (length, utf8(srp(k))),
+                                _F(
+                                    'expected escape sequence of {length:d} hexdecimal '
+                                    'numbers, but found {srp_call!r}',
+                                    length=length,
+                                    srp_call=srp(k),
+                                ),
                                 self.reader.get_mark(),
                             )
                     code = int(self.reader.prefix(length), 16)
-                    chunks.append(unichr(code))
+                    chunks.append(chr(code))
                     srf(length)
                 elif ch in '\n\r\x85\u2028\u2029':
                     self.scan_line_break()
@@ -1456,7 +1476,7 @@ class Scanner(object):
                     raise ScannerError(
                         'while scanning a double-quoted scalar',
                         start_mark,
-                        'found unknown escape character %r' % utf8(ch),
+                        _F('found unknown escape character {ch!r}', ch=ch),
                         self.reader.get_mark(),
                     )
             else:
@@ -1587,10 +1607,22 @@ class Scanner(object):
                 break
 
         token = ScalarToken("".join(chunks), True, start_mark, end_mark)
-        if spaces and spaces[0] == '\n':
-            # Create a comment token to preserve the trailing line breaks.
-            comment = CommentToken("".join(spaces) + '\n', start_mark, end_mark)
-            token.add_post_comment(comment)
+        # getattr provides True so C type loader, which cannot handle comment,
+        # will not make CommentToken
+        if self.loader is not None:
+            comment_handler = getattr(self.loader, 'comment_handling', False)
+            if comment_handler is None:
+                if spaces and spaces[0] == '\n':
+                    # Create a comment token to preserve the trailing line breaks.
+                    comment = CommentToken("".join(spaces) + '\n', start_mark, end_mark)
+                    token.add_post_comment(comment)
+            elif comment_handler is not False:
+                line = start_mark.line + 1
+                for ch in spaces:
+                    if ch == '\n':
+                        self.comments.add_blank_line('\n', 0, line)
+                        line += 1
+
         return token
 
     def scan_plain_spaces(self, indent, start_mark):
@@ -1640,9 +1672,9 @@ class Scanner(object):
         ch = srp()
         if ch != '!':
             raise ScannerError(
-                'while scanning a %s' % (name,),
+                _F('while scanning an {name!s}', name=name),
                 start_mark,
-                "expected '!', but found %r" % utf8(ch),
+                _F("expected '!', but found {ch!r}", ch=ch),
                 self.reader.get_mark(),
             )
         length = 1
@@ -1654,9 +1686,9 @@ class Scanner(object):
             if ch != '!':
                 self.reader.forward(length)
                 raise ScannerError(
-                    'while scanning a %s' % (name,),
+                    _F('while scanning an {name!s}', name=name),
                     start_mark,
-                    "expected '!', but found %r" % utf8(ch),
+                    _F("expected '!', but found {ch!r}", ch=ch),
                     self.reader.get_mark(),
                 )
             length += 1
@@ -1677,7 +1709,7 @@ class Scanner(object):
             or 'A' <= ch <= 'Z'
             or 'a' <= ch <= 'z'
             or ch in "-;/?:@&=+$,_.!~*'()[]%"
-            or ((self.scanner_processing_version > (1, 1)) and ch == "#")
+            or ((self.scanner_processing_version > (1, 1)) and ch == '#')
         ):
             if ch == '%':
                 chunks.append(self.reader.prefix(length))
@@ -1693,9 +1725,9 @@ class Scanner(object):
             length = 0
         if not chunks:
             raise ScannerError(
-                'while parsing a %s' % (name,),
+                _F('while parsing an {name!s}', name=name),
                 start_mark,
-                'expected URI, but found %r' % utf8(ch),
+                _F('expected URI, but found {ch!r}', ch=ch),
                 self.reader.get_mark(),
             )
         return "".join(chunks)
@@ -1712,24 +1744,23 @@ class Scanner(object):
             for k in range(2):
                 if srp(k) not in '0123456789ABCDEFabcdef':
                     raise ScannerError(
-                        'while scanning a %s' % (name,),
+                        _F('while scanning an {name!s}', name=name),
                         start_mark,
-                        'expected URI escape sequence of 2 hexdecimal numbers,'
-                        ' but found %r' % utf8(srp(k)),
+                        _F(
+                            'expected URI escape sequence of 2 hexdecimal numbers,'
+                            ' but found {srp_call!r}',
+                            srp_call=srp(k),
+                        ),
                         self.reader.get_mark(),
                     )
-            if PY3:
-                code_bytes.append(int(self.reader.prefix(2), 16))
-            else:
-                code_bytes.append(chr(int(self.reader.prefix(2), 16)))
+            code_bytes.append(int(self.reader.prefix(2), 16))
             srf(2)
         try:
-            if PY3:
-                value = bytes(code_bytes).decode('utf-8')
-            else:
-                value = unicode(b"".join(code_bytes), 'utf-8')
+            value = bytes(code_bytes).decode('utf-8')
         except UnicodeDecodeError as exc:
-            raise ScannerError('while scanning a %s' % (name,), start_mark, str(exc), mark)
+            raise ScannerError(
+                _F('while scanning an {name!s}', name=name), start_mark, str(exc), mark
+            )
         return value
 
     def scan_line_break(self):
@@ -1762,7 +1793,7 @@ class RoundTripScanner(Scanner):
         while self.need_more_tokens():
             self.fetch_more_tokens()
         self._gather_comments()
-        if bool(self.tokens):
+        if len(self.tokens) > 0:
             if not choices:
                 return True
             for choice in choices:
@@ -1776,13 +1807,13 @@ class RoundTripScanner(Scanner):
         while self.need_more_tokens():
             self.fetch_more_tokens()
         self._gather_comments()
-        if bool(self.tokens):
+        if len(self.tokens) > 0:
             return self.tokens[0]
         return None
 
     def _gather_comments(self):
         # type: () -> Any
-        """combine multiple comment lines"""
+        """combine multiple comment lines and assign to next non-comment-token"""
         comments = []  # type: List[Any]
         if not self.tokens:
             return comments
@@ -1811,7 +1842,7 @@ class RoundTripScanner(Scanner):
         while self.need_more_tokens():
             self.fetch_more_tokens()
         self._gather_comments()
-        if bool(self.tokens):
+        if len(self.tokens) > 0:
             # nprint('tk', self.tokens)
             # only add post comment to single line tokens:
             # scalar, value token. FlowXEndToken, otherwise
@@ -1923,7 +1954,7 @@ class RoundTripScanner(Scanner):
                 if not self.flow_level:
                     self.allow_simple_key = True
                 return comment, start_mark, end_mark
-            if bool(self.scan_line_break()):
+            if self.scan_line_break() != '':
                 start_mark = self.reader.get_mark()
                 if not self.flow_level:
                     self.allow_simple_key = True
@@ -1973,8 +2004,414 @@ class RoundTripScanner(Scanner):
         return Scanner.scan_block_scalar(self, style, rt=rt)
 
 
-# try:
-#     import psyco
-#     psyco.bind(Scanner)
-# except ImportError:
-#     pass
+# commenthandling 2021, differentiatiation not needed
+
+VALUECMNT = 0
+KEYCMNT = 0  # 1
+# TAGCMNT = 2
+# ANCHORCMNT = 3
+
+
+class CommentBase:
+    __slots__ = ('value', 'line', 'column', 'used', 'function', 'fline', 'ufun', 'uline')
+
+    def __init__(self, value, line, column):
+        self.value = value
+        self.line = line
+        self.column = column
+        self.used = ' '
+        info = inspect.getframeinfo(inspect.stack()[3][0])
+        self.function = info.function
+        self.fline = info.lineno
+        self.ufun = None
+        self.uline = None
+
+    def set_used(self, v='+'):
+        self.used = v
+        info = inspect.getframeinfo(inspect.stack()[1][0])
+        self.ufun = info.function
+        self.uline = info.lineno
+
+    def set_assigned(self):
+        self.used = '|'
+
+    def __str__(self):
+        return _F('{value}', value=self.value)
+
+    def __repr__(self):
+        return _F('{value!r}', value=self.value)
+
+    def info(self):
+        return _F(
+            '{name}{used} {line:2}:{column:<2} "{value:40s} {function}:{fline} {ufun}:{uline}',
+            name=self.name,
+            line=self.line,
+            column=self.column,
+            value=self.value + '"',
+            used=self.used,
+            function=self.function,
+            fline=self.fline,
+            ufun=self.ufun,
+            uline=self.uline,
+        )
+
+
+class EOLComment(CommentBase):
+    name = 'EOLC'
+
+    def __init__(self, value, line, column):
+        super().__init__(value, line, column)
+
+
+class FullLineComment(CommentBase):
+    name = 'FULL'
+
+    def __init__(self, value, line, column):
+        super().__init__(value, line, column)
+
+
+class BlankLineComment(CommentBase):
+    name = 'BLNK'
+
+    def __init__(self, value, line, column):
+        super().__init__(value, line, column)
+
+
+class ScannedComments:
+    def __init__(self):
+        self.comments = {}
+        self.unused = []
+
+    def add_eol_comment(self, comment, column, line):
+        # info = inspect.getframeinfo(inspect.stack()[1][0])
+        if comment.count('\n') == 1:
+            assert comment[-1] == '\n'
+        else:
+            assert '\n' not in comment
+        self.comments[line] = retval = EOLComment(comment[:-1], line, column)
+        self.unused.append(line)
+        return retval
+
+    def add_blank_line(self, comment, column, line):
+        # info = inspect.getframeinfo(inspect.stack()[1][0])
+        assert comment.count('\n') == 1 and comment[-1] == '\n'
+        assert line not in self.comments
+        self.comments[line] = retval = BlankLineComment(comment[:-1], line, column)
+        self.unused.append(line)
+        return retval
+
+    def add_full_line_comment(self, comment, column, line):
+        # info = inspect.getframeinfo(inspect.stack()[1][0])
+        assert comment.count('\n') == 1 and comment[-1] == '\n'
+        # if comment.startswith('# C12'):
+        #    raise
+        # this raises in line 2127 fro 330
+        self.comments[line] = retval = FullLineComment(comment[:-1], line, column)
+        self.unused.append(line)
+        return retval
+
+    def __getitem__(self, idx):
+        return self.comments[idx]
+
+    def __str__(self):
+        return (
+            'ParsedComments:\n  '
+            + '\n  '.join(
+                (
+                    _F('{lineno:2} {x}', lineno=lineno, x=x.info())
+                    for lineno, x in self.comments.items()
+                )
+            )
+            + '\n'
+        )
+
+    def last(self):
+        lineno, x = list(self.comments.items())[-1]
+        return _F('{lineno:2} {x}\n', lineno=lineno, x=x.info())
+
+    def any_unprocessed(self):
+        # ToDo: might want to differentiate based on lineno
+        return len(self.unused) > 0
+        # for lno, comment in reversed(self.comments.items()):
+        #    if comment.used == ' ':
+        #        return True
+        # return False
+
+    def unprocessed(self, use=False):
+        while len(self.unused) > 0:
+            first = self.unused.pop(0) if use else self.unused[0]
+            info = inspect.getframeinfo(inspect.stack()[1][0])
+            xprintf('using', first, self.comments[first].value, info.function, info.lineno)
+            yield first, self.comments[first]
+            if use:
+                self.comments[first].set_used()
+
+    def assign_pre(self, token):
+        token_line = token.start_mark.line
+        info = inspect.getframeinfo(inspect.stack()[1][0])
+        xprintf('assign_pre', token_line, self.unused, info.function, info.lineno)
+        gobbled = False
+        while self.unused and self.unused[0] < token_line:
+            gobbled = True
+            first = self.unused.pop(0)
+            xprintf('assign_pre < ', first)
+            self.comments[first].set_used()
+            token.add_comment_pre(first)
+        return gobbled
+
+    def assign_eol(self, tokens):
+        try:
+            comment_line = self.unused[0]
+        except IndexError:
+            return
+        if not isinstance(self.comments[comment_line], EOLComment):
+            return
+        idx = 1
+        while tokens[-idx].start_mark.line > comment_line or isinstance(
+            tokens[-idx], ValueToken
+        ):
+            idx += 1
+        xprintf('idx1', idx)
+        if (
+            len(tokens) > idx
+            and isinstance(tokens[-idx], ScalarToken)
+            and isinstance(tokens[-(idx + 1)], ScalarToken)
+        ):
+            return
+        try:
+            if isinstance(tokens[-idx], ScalarToken) and isinstance(
+                tokens[-(idx + 1)], KeyToken
+            ):
+                try:
+                    eol_idx = self.unused.pop(0)
+                    self.comments[eol_idx].set_used()
+                    xprintf('>>>>>a', idx, eol_idx, KEYCMNT)
+                    tokens[-idx].add_comment_eol(eol_idx, KEYCMNT)
+                except IndexError:
+                    raise NotImplementedError
+                return
+        except IndexError:
+            xprintf('IndexError1')
+            pass
+        try:
+            if isinstance(tokens[-idx], ScalarToken) and isinstance(
+                tokens[-(idx + 1)], (ValueToken, BlockEntryToken)
+            ):
+                try:
+                    eol_idx = self.unused.pop(0)
+                    self.comments[eol_idx].set_used()
+                    tokens[-idx].add_comment_eol(eol_idx, VALUECMNT)
+                except IndexError:
+                    raise NotImplementedError
+                return
+        except IndexError:
+            xprintf('IndexError2')
+            pass
+        for t in tokens:
+            xprintf('tt-', t)
+        xprintf('not implemented EOL', type(tokens[-idx]))
+        import sys
+
+        sys.exit(0)
+
+    def assign_post(self, token):
+        token_line = token.start_mark.line
+        info = inspect.getframeinfo(inspect.stack()[1][0])
+        xprintf('assign_post', token_line, self.unused, info.function, info.lineno)
+        gobbled = False
+        while self.unused and self.unused[0] < token_line:
+            gobbled = True
+            first = self.unused.pop(0)
+            xprintf('assign_post < ', first)
+            self.comments[first].set_used()
+            token.add_comment_post(first)
+        return gobbled
+
+    def str_unprocessed(self):
+        return ''.join(
+            (
+                _F('  {ind:2} {x}\n', ind=ind, x=x.info())
+                for ind, x in self.comments.items()
+                if x.used == ' '
+            )
+        )
+
+
+class RoundTripScannerSC(Scanner):  # RoundTripScanner Split Comments
+    def __init__(self, *arg, **kw):
+        super().__init__(*arg, **kw)
+        assert self.loader is not None
+        # comments isinitialised on .need_more_tokens and persist on
+        # self.loader.parsed_comments
+        self.comments = None
+
+    def get_token(self):
+        # type: () -> Any
+        # Return the next token.
+        while self.need_more_tokens():
+            self.fetch_more_tokens()
+        if len(self.tokens) > 0:
+            if isinstance(self.tokens[0], BlockEndToken):
+                self.comments.assign_post(self.tokens[0])
+            else:
+                self.comments.assign_pre(self.tokens[0])
+            self.tokens_taken += 1
+            return self.tokens.pop(0)
+
+    def need_more_tokens(self):
+        if self.comments is None:
+            self.loader.parsed_comments = self.comments = ScannedComments()
+        if self.done:
+            return False
+        if len(self.tokens) == 0:
+            return True
+        # The current token may be a potential simple key, so we
+        # need to look further.
+        self.stale_possible_simple_keys()
+        if self.next_possible_simple_key() == self.tokens_taken:
+            return True
+        if len(self.tokens) < 2:
+            return True
+        if self.tokens[0].start_mark.line == self.tokens[-1].start_mark.line:
+            return True
+        if True:
+            xprintf('-x--', len(self.tokens))
+            for t in self.tokens:
+                xprintf(t)
+            # xprintf(self.comments.last())
+            xprintf(self.comments.str_unprocessed())
+        self.comments.assign_pre(self.tokens[0])
+        self.comments.assign_eol(self.tokens)
+        return False
+
+    def scan_to_next_token(self):
+        srp = self.reader.peek
+        srf = self.reader.forward
+        if self.reader.index == 0 and srp() == '\uFEFF':
+            srf()
+        start_mark = self.reader.get_mark()
+        # xprintf('current_mark', start_mark.line, start_mark.column)
+        found = False
+        while not found:
+            while srp() == ' ':
+                srf()
+            ch = srp()
+            if ch == '#':
+                comment_start_mark = self.reader.get_mark()
+                comment = ch
+                srf()  # skipt the '#'
+                while ch not in _THE_END:
+                    ch = srp()
+                    if ch == '\0':  # don't gobble the end-of-stream character
+                        # but add an explicit newline as "YAML processors should terminate
+                        # the stream with an explicit line break
+                        # https://yaml.org/spec/1.2/spec.html#id2780069
+                        comment += '\n'
+                        break
+                    comment += ch
+                    srf()
+                # we have a comment
+                if start_mark.column == 0:
+                    self.comments.add_full_line_comment(
+                        comment, comment_start_mark.column, comment_start_mark.line
+                    )
+                else:
+                    self.comments.add_eol_comment(
+                        comment, comment_start_mark.column, comment_start_mark.line
+                    )
+                    comment = ""
+                # gather any blank lines or full line comments following the comment as well
+                self.scan_empty_or_full_line_comments()
+                if not self.flow_level:
+                    self.allow_simple_key = True
+                return
+            if bool(self.scan_line_break()):
+                # start_mark = self.reader.get_mark()
+                if not self.flow_level:
+                    self.allow_simple_key = True
+                self.scan_empty_or_full_line_comments()
+                return None
+                ch = srp()
+                if ch == '\n':  # empty toplevel lines
+                    start_mark = self.reader.get_mark()
+                    comment = ""
+                    while ch:
+                        ch = self.scan_line_break(empty_line=True)
+                        comment += ch
+                    if srp() == '#':
+                        # empty line followed by indented real comment
+                        comment = comment.rsplit('\n', 1)[0] + '\n'
+                    _ = self.reader.get_mark()  # gobble end_mark
+                    return None
+            else:
+                found = True
+        return None
+
+    def scan_empty_or_full_line_comments(self):
+        blmark = self.reader.get_mark()
+        assert blmark.column == 0
+        blanks = ""
+        comment = None
+        mark = None
+        ch = self.reader.peek()
+        while True:
+            # nprint('ch', repr(ch), self.reader.get_mark().column)
+            if ch in '\r\n\x85\u2028\u2029':
+                if self.reader.prefix(2) == '\r\n':
+                    self.reader.forward(2)
+                else:
+                    self.reader.forward()
+                if comment is not None:
+                    comment += '\n'
+                    self.comments.add_full_line_comment(comment, mark.column, mark.line)
+                    comment = None
+                else:
+                    blanks += '\n'
+                    self.comments.add_blank_line(blanks, blmark.column, blmark.line)
+                blanks = ""
+                blmark = self.reader.get_mark()
+                ch = self.reader.peek()
+                continue
+            if comment is None:
+                if ch in ' \t':
+                    blanks += ch
+                elif ch == '#':
+                    mark = self.reader.get_mark()
+                    comment = '#'
+                else:
+                    # print('breaking on', repr(ch))
+                    break
+            else:
+                comment += ch
+            self.reader.forward()
+            ch = self.reader.peek()
+
+    def scan_block_scalar_ignored_line(self, start_mark):
+        # type: (Any) -> Any
+        # See the specification for details.
+        srp = self.reader.peek
+        srf = self.reader.forward
+        prefix = ''
+        comment = None
+        while srp() == ' ':
+            prefix += srp()
+            srf()
+        if srp() == '#':
+            comment = ''
+            mark = self.reader.get_mark()
+            while srp() not in _THE_END:
+                comment += srp()
+                srf()
+            comment += '\n'
+        ch = srp()
+        if ch not in _THE_END:
+            raise ScannerError(
+                'while scanning a block scalar',
+                start_mark,
+                _F('expected a comment or a line break, but found {ch!r}', ch=ch),
+                self.reader.get_mark(),
+            )
+        if comment is not None:
+            self.comments.add_eol_comment(comment, mark.column, mark.line)
+        self.scan_line_break()
+        return None
